			.include "arch-sio.inc"
			.include "arch.inc"
			.include "ring_buf.inc"

			.area CODE2

			SIO_BUFFER_GUARD = 50 	; number of unused bytes in buffer to guard 
						; against receiver overruns

			RX_FLAG_DTR = 0 ; flag is set in zero page flags when dtr is blocked
			TX_FLAG_IDLE = 0 ; flag is unset in zero page flags when transmit interrupt finds buffer empty

sio_init::		push 	AF
			push 	BC
			push	HL

			ld	A, #0H00
			ld	HL, #ARCH_Z_SIO_RX_HEAD
			ld	B, #6
1$:			ld	(HL), A			; zero all 
			inc	HL
			djnz	1$

			ld 	C, #ARCH_IO_SIO_ACON
			ld	A, #0H18		; reset channel
			out	(C), A

			ld	A, #0H04		; point to WR4
			out	(C), A
			ld	A, #0HC4		; x64 clock, 1 stop, no partiy
			out	(C), A

			ld	A, #0H03		; point to WR3
			out	(C), A
			ld	A, #0HE1		; 8 bit rx, rx enable; auto enables
			out	(C), A

			ld	A, #0H05		; point to WR5
			out	(C), A
			ld	A, #0HEA		; 8 bit tx, tx enable, rts, dtr
			out	(C), A

			ld	A, #0H01		; point to WR1
			out	(C), A			
			ld	A, #0H1A		; enable interrupts for receive
							; parity error ignored, and tx and
							; not dcd/cts
			out	(C), A

			pop	HL
			pop	BC
			pop	AF
			ret

sio_send::		push AF
			push HL
			push IX			
			ld	HL, #ARCH_MEM_SIO_BUF_TX
			ld	IX, #ARCH_Z_SIO_TX_HEAD
1$:			di
			bit	TX_FLAG_IDLE, 2(IX)	; see if tx is idle
			jr	NZ, 2$			; its not let the interrupt handler send next char
			set	TX_FLAG_IDLE, 2(IX)	; interrupt routine ran out of chars, set it back off again
			out	(ARCH_IO_SIO_ADATA), A  ; send the character
			ei
			jr	send_fin
2$:			call	rbuf_write		; write to buffer
			ei
			nop
			jr	Z, 1$			; loop until buffer is freed by interrupts						
send_fin:		pop	IX
			pop	HL
			pop	AF
			ret			
			
			

sio_send_asciiz::	push	HL
			push	AF
2$:			ld	A, (HL)
			and	A
			jr	Z, 3$
			call	sio_send
			inc	HL
			jr	2$
3$:
			pop	AF
			pop	HL
			ret

			; this returns immediately with Z set and A clear if no character ready
sio_recv::		push HL
			push IX			
			ld	HL, #ARCH_MEM_SIO_BUF_RX
			ld	IX, #ARCH_Z_SIO_RX_HEAD
			di
			call	rbuf_read
			jr	Z, 1$
	
			; check to see if there is now room in the input buffer and if
			; there is reenable DTR
			push	AF

			bit	RX_FLAG_DTR, 2(IX)
			jr	Z, 2$

			; set up rbuf pointers and write to buffer
			call	rbuf_count
			cp	#254 - SIO_BUFFER_GUARD
			jr	NC, 2$

			; disable DTR to deter further characters
			ld	A, #0H05		; point to WR5
			out	(ARCH_IO_SIO_ACON), A
			ld	A, #0HEA		; 8 bit tx, tx enable, rts, dtr
			out	(ARCH_IO_SIO_ACON), A
			res	RX_FLAG_DTR, 2(IX)
			
2$:			pop	AF
1$:			ei
			pop	IX
			pop	HL
			ret			
			
			; return no of chars waiting in recv buffer in A
sio_recv_count::	push 	HL
			push 	IX			
			ld	HL, #ARCH_MEM_SIO_BUF_RX
			ld	IX, #ARCH_Z_SIO_RX_HEAD
			call	rbuf_count
			pop	IX
			pop	HL
			ret


sio_recv_none_ready:	xor	A
			ret
			
			; does not preserve flags!
sio_int_check:		
			ld	C, #ARCH_IO_SIO_ACON
			in	A, (C)
			and	#0H02			; check for any SIO interrupts
			jr	Z, no_int	
			in	A, (C)
			and	#0H04			; check if transmit buffer empty
			jr	Z, no_tx
			ld	A, #0H28		; reset interrupt pending flag
			out	(C), A
			ld	HL, #ARCH_MEM_SIO_BUF_TX
			ld	IX, #ARCH_Z_SIO_TX_HEAD
			bit	TX_FLAG_IDLE, 2(IX)
			jr	Z, no_tx
			call	rbuf_read
			jr	Z, go_idle
			out	(ARCH_IO_SIO_ADATA), A
			jr	no_tx
go_idle:		res	TX_FLAG_IDLE, 2(IX)
no_tx:			
			ld	HL, #ARCH_MEM_SIO_BUF_RX
			ld	IX, #ARCH_Z_SIO_RX_HEAD
recv_again:
			; check if there is a received character error
			ld	A, #0H01
			out	(C), A
			in	A, (C)
			and	#0H70
			jr	Z, no_recv_err
			; just reset the error flags - we're not really bothered !!!!????
			ld	A, #0H30
			out	(C), A
no_recv_err:
			; check if there is a received character
			in	A, (C)
			and	#0H01
			jr	Z, no_recv
			in	A, (ARCH_IO_SIO_ADATA)
			
			; set up rbuf pointers and write to buffer
			call	rbuf_write
			cp	A, #SIO_BUFFER_GUARD
			jr	NC, no_recv
			
			; disable DTR to deter further characters
			ld	A, #0H05		; point to WR5
			out	(C), A
			ld	A, #0H6A		; 8 bit tx, tx enable, rts, dtr
			out	(C), A
			set	RX_FLAG_DTR, 2(IX)
			jr	recv_again		; is there another char?
no_recv:		

no_int:
			ret
