		.include "arch.inc"

		.area BIOSREGS
AUTO:		.blkb 	1
AUTOFLAG:	.blkb 	1
PARAMS:		.blkw	1
EPARAM:		.blkb	1
STATUS:		.blkw	1
USRSTACK:	.blkw	1
IOBYTE:		.blkb	1
CDISK:		.blkb	1
CHARBUF:	.blkb	1
OUTFLAG:	.blkb	1
PRTFLAG:	.blkb   1
CURPOS:		.blkw	1
STARTING:	.blkw	1
monitor:	.blkb	1
ENDBIOSREGS:	.blkb	1
;
;   Set control character equates.
;
CNTRLC	=	3		;control-c
CNTRLE	=	0H05		;control-e
BS	=	0H08		;backspace
TAB	=	0H09		;tab
LF	=	0H0A		;line feed
FF	=	0H0C		;form feed
CR	=	0H0D		;carriage return
CNTRLP	=	0H10		;control-p
CNTRLR	=	0H12		;control-r
CNTRLS	=	0H13		;control-s
CNTRLU	=	0H15		;control-u
CNTRLX	=	0H18		;control-x
CNTRLZ	=	0H1A		;control-z (end-of-file mark)
DEL	=	0H7F		;rubout


		.area CODE2
		.include "lcd-lib.inc"
		.include "arch-sio.inc"
		.include "arch.inc"
		.include "int.inc"
		.include "str-utils.inc"
		.globl	main

		STKAREA = ARCH_MEM_BIOSSTACK

bios_monitor::	push	HL
		ld	HL, #monitor
		xor	(HL)
		ld	(HL), A
		pop	HL
		ret

bios_boot::	di
		; set up the rom

		ld	A, #0
		ld	(monitor), A

		ld	A, #0hC0
		ld	C, #ARCH_IO_ROMAD
		
		out	(C), A		; disable ROM at RAM0
					; select ROM bank 0


		ld	A, #0HDB
		; clear memory - overkill?
		ld	BC, #ARCH_MEM_TOT
		ld	HL, #0
2$:		ld	(HL), A
		inc	HL
		dec	C
		jr	NZ, 2$
		djnz	2$

		xor	A

		ld	B, #(ENDBIOSREGS - AUTO)
		ld	HL, #AUTO
1$:		ld	(HL), A
		inc	HL
		djnz	1$

bios_wboot::	di
		ld	SP, #ARCH_MEM_BIOSSTACK
		call	lcd_init
		call	sio_init
		call	int_init
		
		ld	A, #0HC3
		ld	(0), A
		ld	HL, #WBOOT
		ld	(1), HL
		ld	(5), A
		ld	HL, #FBASE	
		ld	(6), HL
		; make a dummy jump point to ROM for programs to help guess size of user memory...
		ld	(FBASE), A
		ld	HL, #BDOS
		ld	(FBASE + 1), HL
		jp	main			


bios_const:	call	sio_recv_count
		cp	#0
		ret	Z
		ld	A, #FF
		ret
		
bios_conin:	call	sio_recv
		jr	Z, bios_conin
		ret

bios_conout:	ld	A, C
		jp	sio_send

bios_list:	ld	A, C
		jp	lcd_send_char

bios_listst:	ld	A, #0HFF
		ret

bios_punch:	ld	A, C
		jp	lcd_send_char

bios_reader:	ld	A, #0H1A ; EOF
		ret

bios_home:	
bios_settrk:
bios_seldsk:
bios_setsec:
bios_setdma:
bios_read:
bios_write:
bios_prstat:
bios_sectrn:
		ret


;**************************************************************
;*
;*                    B D O S   E N T R Y
;*
;**************************************************************
;
;
;   Entry into bdos. (DE) or (E) are the parameters passed. The
; function number desired is in register (C).
;
BDOS:	
FBASE1:	EX	DE,HL		;save the (DE) parameters.
	LD	(PARAMS),HL
	EX	DE,HL
	LD	A,E		;and save register (E) in particular.
	LD	(EPARAM),A
	LD	HL,#0
	LD	(STATUS),HL	;clear return status.
	ADD	HL,SP
	LD	(USRSTACK),HL	;save users stack pointer.
	LD	SP,#STKAREA	;and set our own.
	XOR	A		;clear auto select storage space.
	LD	(AUTOFLAG),A
	LD	(AUTO),A
	
	ld	A, (monitor)
	bit	0, A
	jr	Z, 2$

	push	BC
	ld	BC, #0H0001
	call	lcd_gotoxy

	ld	A, #32
	ld	B, #8
1$:	call	lcd_send_char
	djnz	1$

	ld	BC, #0H0001
	call	lcd_gotoxy

	pop	BC
	push	BC				; get C back for display
	ld	HL, #ARCH_MEM_USER_TOP + 0H100 ; !!!!!!! ONLY FOR NOW WHILE USER MEMORY IS LOW
	push	HL
	ld	A, C
	call	str_btoah
	pop	HL
	call	lcd_send_asciiz
	ld	A, #" "
	call	lcd_send_char
	pop	BC

2$:	LD	HL,#GOBACK	;set return address.
	PUSH	HL
	LD	A,C		;get function number.
	CP	#NFUNCTS		;valid function number?
	RET	NC
	LD	C,E		;keep single register function here.
	LD	HL,#FUNCTNS	;now look thru the function table.
	LD	E,A
	LD	D,#0		;(DE)=function number.
	ADD	HL,DE
	ADD	HL,DE		;(HL)=(start of table)+2*(function number).
	LD	E,(HL)
	INC	HL
	LD	D,(HL)		;now (DE)=address for this function.
	LD	HL,(PARAMS)	;retrieve parameters.
	EX	DE,HL		;now (DE) has the original parameters.
	JP	(HL)		;execute desired function.
;
;   BDOS function jump table.
;
		NFUNCTS =	13		;number of functions in followin table.
;
FUNCTNS:.dw	WBOOT,GETCON,OUTCON,GETRDR,PUNCH,LIST,DIRCIO,GETIOB
	.dw	SETIOB,PRTSTR,RDBUFF,GETCSTS,GETVER


;
;   Get here to return to the user.
;
GOBACK:	
	;!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	ld	A, (monitor)
	bit	0, A
	jr	Z, 2$

	ld	A, #0H7F
	call	lcd_send_char

2$:	LD	A,(AUTO)	;was auto select activated?
	OR	A
	JP	Z,GOBACK1
	LD	HL,(PARAMS)	;yes, but was a change made?
	LD	(HL),#0		;(* reset first byte of fcb *)
	LD	A,(AUTOFLAG)
	OR	A
	JP	Z,GOBACK1
	LD	(HL),A		;yes, reset first byte properly.
	LD	(EPARAM),A
GOBACK1:LD	HL,(USRSTACK)	;reset the users stack pointer.
	LD	SP,HL
	LD	HL,(STATUS)	;get return status.
	LD	A,L		;force version 1.4 compatability.
	LD	B,H
	RET			;and go back to user.


GETCHAR:LD	HL,#CHARBUF	;check character buffer.
	LD	A,(HL)		;anything present already?
	LD	(HL), #0		;...either case clear it.
	OR	A
	RET	NZ		;yes, use it.
	JP	CONIN		;nope, go get a character responce.
;
;   Input and echo a character.
;
GETECHO:CALL	GETCHAR		;input a character.
	CALL	CHKCHAR		;carriage control?
	RET	C		;no, a regular control char so don't echo.
	PUSH	AF		;ok, save character now.
	LD	C,A
	CALL	OUTCON		;and echo it.
	POP	AF		;get character and return.
	RET	
;
;   Check character in (A). Set the zero flag on a carriage
; control character and the carry flag on any other control
; character.
;
CHKCHAR:CP	#CR		;check for carriage return, line feed, backspace,
	RET	Z		;or a tab.
	CP	#LF
	RET	Z
	CP	#TAB
	RET	Z
	CP	#BS
	RET	Z
	CP	#" "		;other control char? Set carry flag.
	RET	
;
;   Check the console during output. Halt on a control-s, then
; reboot on a control-c. If anything else is ready, clear the
; zero flag and return (the calling routine may want to do
; something).
;
CKCONSOL: LD	A,(CHARBUF)	;check buffer.
	OR	A		;if anything, just return without checking.
	JP	NZ,CKCON2
	CALL	CONST		;nothing in buffer. Check console.
	AND	#0H01		;look at bit 0.
	RET	Z		;return if nothing.
	CALL	CONIN		;ok, get it.
	CP	#CNTRLS		;if not control-s, return with zero cleared.
	JP	NZ,CKCON1
	CALL	CONIN		;halt processing until another char
	CP	#CNTRLC		;is typed. Control-c?
	JP	Z,0		;yes, reboot now.
	XOR	A		;no, just pretend nothing was ever ready.
	RET	
CKCON1:	LD	(CHARBUF),A	;save character in buffer for later processing.
CKCON2:	LD	A,#1		;set (A) to non zero to mean something is ready.
	RET	
;
;   Output (C) to the screen. If the printer flip-flop flag
; is set, we will send character to printer also. The console
; will be checked in the process.
;
OUTCHAR:LD	A,(OUTFLAG)	;check output flag.
	OR	A		;anything and we won't generate output.
	JP	NZ,OUTCHR1
	PUSH	BC
	CALL	CKCONSOL	;check console (we don't care whats there).
	POP	BC
	PUSH	BC
	CALL	CONOUT		;output (C) to the screen.
	POP	BC
	PUSH	BC
	LD	A,(PRTFLAG)	;check printer flip-flop flag.
	OR	A
	CALL	NZ,LIST		;print it also if non-zero.
	POP	BC
OUTCHR1:LD	A,C		;update cursors position.
	LD	HL,#CURPOS
	CP	#DEL		;rubouts don't do anything here.
	RET	Z
	INC	(HL)		;bump line pointer.
	CP	#" "		;and return if a normal character.
	RET	NC
	DEC	(HL)		;restore and check for the start of the line.
	LD	A,(HL)
	OR	A
	RET	Z		;ingnore control characters at the start of the line.
	LD	A,C
	CP	#BS		;is it a backspace?
	JP	NZ,OUTCHR2
	DEC	(HL)		;yes, backup pointer.
	RET	
OUTCHR2:CP	#LF		;is it a line feed?
	RET	NZ		;ignore anything else.
	LD	(HL),#0		;reset pointer to start of line.
	RET	


;
;   Output (A) to the screen. If it is a control character
; (other than carriage control), use ^x format.
;
SHOWIT:	LD	A,C
	CALL	CHKCHAR		;check character.
	JP	NC,OUTCON	;not a control, use normal output.
	PUSH	AF
	LD	C,#"^"		;for a control character, preceed it with '^'.
	CALL	OUTCHAR
	POP	AF
	OR	#"@"		;and then use the letter equivelant.
	LD	C,A

;
;   Function to output (C) to the console device and expand tabs
; if necessary.
;
OUTCON:	LD	A,C
	CP	#TAB		;is it a tab?
	JP	NZ,OUTCHAR	;use regular output.
OUTCON1:LD	C,#" "		;yes it is, use spaces instead.
	CALL	OUTCHAR
	LD	A,(CURPOS)	;go until the cursor is at a multiple of 8

	AND	#0H07		;position.
	JP	NZ,OUTCON1
	RET	


;
;   Function to get a character from the console device.
;
GETCON:	CALL	GETECHO		;get and echo.
	JP	SETSTAT		;save status and return.
;
;   Function to get a character from the tape reader device.
;
GETRDR:	CALL	READER		;get a character from reader, set status and return.
	JP	SETSTAT
;
;  Function to perform direct console i/o. If (C) contains (FF)
; then this is an input request. If (C) contains (FE) then
; this is a status request. Otherwise we are to output (C).
;
DIRCIO:	LD	A,E		;test for (FF).				!!!! CHANGED BY DOM!
	INC	A
	JP	Z,DIRC1
	INC	A		;test for (FE).
	JP	Z,CONST
	JP	CONOUT		;just output (C).
DIRC1:	CALL	CONST		;this is an input request.
	OR	A
	JP	Z,GOBACK1	;not ready? Just return (directly).
	CALL	CONIN		;yes, get character.
	JP	SETSTAT		;set status and return.
;
;   Function to return the i/o byte.
;
GETIOB:	LD	A,(IOBYTE)
	JP	SETSTAT
;
;   Function to set the i/o byte.
;
SETIOB:	LD	HL,#IOBYTE
	LD	(HL),C
	RET	
;
;   Function to print the character string pointed to by (DE)
; on the console device. The string ends with a '$'.
;
PRTSTR:	EX	DE,HL
	LD	C,L
	LD	B,H		;now (BC) points to it.
	JP	PRTMESG
;
;   Function to interigate the console device.
;
GETCSTS:CALL	CKCONSOL
;
;   Get here to set the status and return to the cleanup
; section. Then back to the user.
;
SETSTAT:LD	(STATUS),A
RTN:	RET	
;
;   Set the status to 1 (read or write error code).
;
IOERR1:	LD	A,#1
	JP	SETSTAT

;
;   Echo a backspace character. Erase the prevoius character
; on the screen.
;
BACKUP:	CALL	BACKUP1		;backup the screen 1 place.
	LD	C,#" "	;then blank that character.
	CALL	CONOUT
BACKUP1:LD	C,#BS		;then back space once more.
	JP	CONOUT
;
;   Signal a deleted line. Print a '#' at the end and start
; over.
;
NEWLINE:LD	C,#"#"
	CALL	OUTCHAR		;print this.
	CALL	OUTCRLF		;start new line.
NEWLN1:	LD	A,(CURPOS)	;move the cursor to the starting position.
	LD	HL,#STARTING
	CP	(HL)
	RET	NC		;there yet?
	LD	C,#" "
	CALL	OUTCHAR		;nope, keep going.
	JP	NEWLN1

;
;   Output a (cr) (lf) to the console device (screen).
;
OUTCRLF:LD	C,#CR
	CALL	OUTCHAR
	LD	C,#LF
	JP	OUTCHAR

;
;   Print message pointed to by (BC). It will end with a '$'.
;
PRTMESG:LD	A,(BC)		;check for terminating character.
	CP	#"$"
	RET	Z
	INC	BC
	PUSH	BC		;otherwise, bump pointer and print it.
	LD	C,A
	CALL	OUTCON
	POP	BC
	JP	PRTMESG


;
;   Function to execute a buffered read.
;
RDBUFF:	LD	A,(CURPOS)	;use present location as starting one.
	LD	(STARTING),A
	LD	HL,(PARAMS)	;get the maximum buffer space.
	LD	C,(HL)
	INC	HL		;point to first available space.
	PUSH	HL		;and save.
	LD	B,#0		;keep a character count.
RDBUF1:	PUSH	BC
	PUSH	HL
RDBUF2:	CALL	GETCHAR		;get the next input character.
	AND	#0H7F		;strip bit 7.
	POP	HL		;reset registers.
	POP	BC
	CP	#CR		;en of the line?
	JP	Z,RDBUF17
	CP	#LF
	JP	Z,RDBUF17
	CP	#BS		;how about a backspace?
	JP	NZ,RDBUF3
	LD	A,B		;yes, but ignore at the beginning of the line.
	OR	A
	JP	Z,RDBUF1
	DEC	B		;ok, update counter.
	LD	A,(CURPOS)	;if we backspace to the start of the line,
	LD	(OUTFLAG),A	;treat as a cancel (control-x).
	JP	RDBUF10
RDBUF3:	CP	#DEL		;user typed a rubout?
	JP	NZ,RDBUF4
	LD	A,B		;ignore at the start of the line.
	OR	A
	JP	Z,RDBUF1
	LD	A,(HL)		;ok, echo the prevoius character.
	DEC	B		;and reset pointers (counters).
	DEC	HL
	JP	RDBUF15
RDBUF4:	CP	#CNTRLE		;physical end of line?
	JP	NZ,RDBUF5
	PUSH	BC		;yes, do it.
	PUSH	HL
	CALL	OUTCRLF
	XOR	A		;and update starting position.
	LD	(STARTING),A
	JP	RDBUF2
RDBUF5:	CP	#CNTRLP		;control-p?
	JP	NZ,RDBUF6
	PUSH	HL		;yes, flip the print flag filp-flop byte.
	LD	HL,#PRTFLAG
	LD	A,#1		;PRTFLAG=1-PRTFLAG
	SUB	(HL)
	LD	(HL),A
	POP	HL
	JP	RDBUF1
RDBUF6:	CP	#CNTRLX		;control-x (cancel)?
	JP	NZ,RDBUF8
	POP	HL
RDBUF7:	LD	A,(STARTING)	;yes, backup the cursor to here.
	LD	HL,#CURPOS
	CP	(HL)
	JP	NC,RDBUFF	;done yet?
	DEC	(HL)		;no, decrement pointer and output back up one space.
	CALL	BACKUP
	JP	RDBUF7
RDBUF8:	CP	#CNTRLU		;cntrol-u (cancel line)?
	JP	NZ,RDBUF9
	CALL	NEWLINE		;start a new line.
	POP	HL
	JP	RDBUFF
RDBUF9:	CP	#CNTRLR		;control-r?
	JP	NZ,RDBUF14
RDBUF10:PUSH	BC		;yes, start a new line and retype the old one.
	CALL	NEWLINE
	POP	BC
	POP	HL
	PUSH	HL
	PUSH	BC
RDBUF11:LD	A,B		;done whole line yet?
	OR	A
	JP	Z,RDBUF12
	INC	HL		;nope, get next character.
	LD	C,(HL)
	DEC	B		;count it.
	PUSH	BC
	PUSH	HL
	CALL	SHOWIT		;and display it.
	POP	HL
	POP	BC
	JP	RDBUF11
RDBUF12:PUSH	HL		;done with line. If we were displaying
	LD	A,(OUTFLAG)	;then update cursor position.
	OR	A
	JP	Z,RDBUF2
	LD	HL,#CURPOS	;because this line is shorter, we must
	SUB	(HL)		;back up the cursor (not the screen however)
	LD	(OUTFLAG),A	;some number of positions.
RDBUF13:CALL	BACKUP		;note that as long as (OUTFLAG) is non
	LD	HL,#OUTFLAG	;zero, the screen will not be changed.
	DEC	(HL)
	JP	NZ,RDBUF13
	JP	RDBUF2		;now just get the next character.
;
;   Just a normal character, put this in our buffer and echo.
;
RDBUF14:INC	HL
	LD	(HL),A		;store character.
	INC	B		;and count it.
RDBUF15:PUSH	BC
	PUSH	HL
	LD	C,A		;echo it now.
	CALL	SHOWIT
	POP	HL
	POP	BC
	LD	A,(HL)		;was it an abort request?
	CP	#CNTRLC		;control-c abort?
	LD	A,B
	JP	NZ,RDBUF16
	CP	#1		;only if at start of line.
	JP	Z,0
RDBUF16:CP	C		;nope, have we filled the buffer?
	JP	C,RDBUF1
RDBUF17:POP	HL		;yes end the line and return.
	LD	(HL),B
	LD	C,#CR
	JP	OUTCHAR		;output (cr) and return.


;
;   Function to return the current cp/m version number.
;
GETVER:	LD	A,#0H022		;version 2.2
	JP	SETSTAT


;
;**************************************************************
;*
;*        B I O S   J U M P   T A B L E
;*
;**************************************************************
;
BOOT::	JP	bios_boot
WBOOT:	JP	bios_wboot
CONST:	JP	bios_const
CONIN:	JP	bios_conin
CONOUT:	JP	bios_conout
LIST:	JP	bios_list
PUNCH:	JP	bios_punch
READER:	JP	bios_reader
HOME:	JP	bios_home
SELDSK:	JP	bios_seldsk
SETTRK:	JP	bios_settrk
SETSEC:	JP	bios_setsec
SETDMA:	JP	bios_setdma
READ:	JP	bios_read
WRITE:	JP	bios_write
PRSTAT:	JP	bios_prstat
SECTRN:	JP	bios_sectrn


