			.area CODE2

			.include "lcd-lib.inc"
			.include "arch-sio.inc"
			.include "int.inc"
			.include "arch.inc"
			.include "str-utils.inc"
			.include "bios.inc"
			.globl bios_monitor

			WORKSPACE = ARCH_MEM_USER_TOP - 0H0300

			REGSAVE = WORKSPACE
			REGSAVE_AF = REGSAVE + 0H00
			REGSAVE_BC = REGSAVE + 0H02
			REGSAVE_DE = REGSAVE + 0H04
			REGSAVE_HL = REGSAVE + 0H06
			REGSAVE_IX = REGSAVE + 0H08
			REGSAVE_IY = REGSAVE + 0H0A
			REGSAVE_SP = REGSAVE + 0H0C
			REG_STR_BUF = REGSAVE + 0H0E	; this is 16 chars long
			
			DUMP_HL_SAVE = REGSAVE + 0H1E
			DUMP_BC_SAVE = REGSAVE + 0H20
			DUMP_PTR = REGSAVE + 0H22

			READ_LEN = REGSAVE + 0H24
			READ_ERR = REGSAVE + 0H26
			READ_STR_BUF = REGSAVE + 0H28 ; 16 bytes for this

			LOAD_ROM_ROM = REGSAVE + 0H38

			STR_BUF= WORKSPACE + 0H100	; 256 bytes for this



main::			ld	SP, #ARCH_MEM_USER_TOP

			ld	HL, #sz_welcome
			call	lcd_send_asciiz
			call	sio_send_asciiz

			ld	BC, #0H0001
			call	lcd_gotoxy


			ld	DE, #s$message
			ld	C, #0H09
			call	0H0005

			
			xor 	A		; reset FCBs and buffer
			ld	HL, #0H5C
			ld	B, #0H20
1$:			ld	(HL), A
			inc	HL
			djnz	1$

			ld	A, #" "
			ld	HL, #0H5D
			ld	B, #11
2$:			ld	(HL), A
			inc	HL
			djnz	2$
			ld	HL, #0H6D
			ld	B, #11
3$:			ld	(HL), A
			inc	HL
			djnz	3$
			
			xor	A
			ld	(0H80), A	; reset buffer area

			call	dump_regs
			ld	HL, #0
			ld	(DUMP_PTR), HL
			call	dump_page


main_command_loop:	ld	A,#"-"			
			call	conout
			call	read_edit
			call	crlf
			jr	Z, main_command_loop
			ld	BC, #0H0001
			call	lcd_gotoxy
			call	lcd_send_asciiz
			
main_decode_command:	ld	IX, #cmd_table
main_decode_loop:	ld	A, 0(IX)
			or	A
			jr	Z, main_decode_error
			cp	(HL)
			jr	Z, main_decode_found
			xor	#0H20			; try opposite case
			cp	(HL)
			jr	NZ,main_decode_skip
main_decode_found:	inc	HL
			ld	B, 2(IX)
			ld	C, 1(IX)
			push	BC			; jump to routine
			ret	

main_decode_skip:	ld	BC, #5
			add	IX, BC
			jr	main_decode_loop

main_decode_error:	; print command up to offending char
			push	HL
			ld	DE, #STR_BUF + 1
			and	A
			sbc	HL, DE
			ld	B, L
			ld	HL, #STR_BUF + 2
$1:			ld	A, (HL)
			call	sio_send	
			inc	HL
			djnz	$1
			ld	A, #"?"
			call	conout
			call	conout_asciiz
			call	crlf
			jp	main_command_loop


read_edit:		ld	A, #0HFE		; setup buffer
			ld	HL, #STR_BUF
			push	HL
			ld	(STR_BUF), A
			ex	DE, HL
			ld	C, #0H0A
			call	5			; call bios's routine
			pop	HL
			inc	HL
			ld	A, (HL)
			or	A
			ret	Z			; check length and return if empty with Z set
			push	AF
			inc	HL
			push	HL
			add	L
			ld	L, A
			jr	NC, 1$
			inc	H
1$:			xor	A
			ld	(HL), A			; store zero terminator
			pop	HL			; return with z clear (pushed on stack after RET Z)
			pop	AF 
			ret

read_line:		; like the routine above but without each or editing
			; any < 32 character returns
			; skips CR / LF /TAB / SPACE at start
			push	HL
			push	IX
			ld	IX, #STR_BUF
			ld	HL, #STR_BUF + 2
			ld	A, #0HFE
			ld	0(IX), A
			xor	A
			ld	1(IX), A
			
1$:			call 	condirin
			cp	#0
			jr	Z, 1$
			cp	#32
			jr	Z, 1$
			cp	#14
			jr	Z, 1$
			cp	#13
			jr	Z, 1$
			cp	#11
			jr	Z, 1$
			cp	#3
			jp	Z, main

2$:			cp	#32
			jr	C, 3$
			ld	(HL), A
			inc	HL
			inc	1(IX)
			dec	0(IX)
			jr	Z, 3$
4$:			call	condirin
			cp	#0
			jr	Z, 4$
			jr	2$
3$:			xor	A
			ld	(HL), A
			ld	A, 1(IX)
			cp 	#0
			pop	IX
			pop	HL
			ret

dump_regs:		push	AF
			push	HL
			push	BC
			push	DE

			ld 	(REGSAVE_HL), HL	; save HL first
			ld	HL, #8
			add	HL, SP
			ld	(REGSAVE_SP), HL	; save SP artificially add 8 to it to counter our saves
			push	AF
			pop	HL
			ld	(REGSAVE_AF), HL	; save AF
			ld	(REGSAVE_BC), BC	; save BC
			ld	(REGSAVE_DE), DE	; save DE
			ld	(REGSAVE_IX), IX	; save DE
			ld	(REGSAVE_IY), IY	; save DE

			call	crlf			; new line

			ld	B, #7			; seven regs to write out
			ld	HL, #REGSAVE		; save area
			ld	DE, #sz_AF		; the register labels
1$:			EX	DE, HL			; point HL at reg label
			call	conout_asciiz		; print it
			inc	HL			; move HL on 5
			inc	HL
			inc	HL
			inc	HL
			inc	HL
			EX	DE, HL			; swap HL back to point at reg saves
			push	BC		
			ld	B, (HL)
			inc	HL
			ld	C, (HL)
			inc	HL
			call	hex2			; print reg contents in hex
			pop	BC
			djnz	1$			; loop

			call	crlf			; new line

			ld	B, #8			; 8 flags
			ld	A, (REGSAVE_AF)		; load flags
			ld	HL, #sz_FLAGS		; load flag labels pointer
2$:			rla				; C = flag
			push	AF			; save AF
			ld	A, (HL)			; load flag label
			jr	C, 3$			
			or	#0H20			; lower case if flag not set
3$:			call	conout			; print char
			ld	A, #" "			; print space
			call	conout	
			pop	AF			; get back A
			inc	HL			; next flag label
			djnz	2$			; loop

			call	crlf			; new line

			pop	DE
			pop	BC
			pop	HL
			pop	AF
			ret

dump_page:		push	BC
			ld	BC, #0H100
			call	dumpN
			pop	BC
			ret

dumpN:			; dump page at HL to sio
			push	AF
			push	BC
			push	HL
			ld	HL, (DUMP_PTR)

1$:			push	BC
			call	crlf			; new line
			ld	C, L
			ld	B, H
			call	hex2
			pop	BC
			push	HL
			ld	HL, #sz_SEP
			call	conout_asciiz
			pop	HL
			ld	(DUMP_HL_SAVE), HL	; store pointer for ascii dump
			ld	(DUMP_BC_SAVE), BC	; store count for ascii dump
2$:			ld	A, (HL)
			call	hex1
			ld	A, #" "
			call	conout
			inc	HL
			dec	C
			jr	Z, 3$
			ld	A, C
			and	#0H0F
			jr	NZ, 2$
			call	4$
			jr	1$
3$:			djnz	4$
			call	4$

			call	crlf			; new line

			ld	(DUMP_PTR), HL
			pop	HL
			pop	BC
			pop	AF
			ret

4$:			;dump this lines ascii
			push	HL
			push	BC
			ld	HL, #sz_SEP
			call	conout_asciiz
			ld	HL, (DUMP_HL_SAVE)
			ld	BC, (DUMP_BC_SAVE)
5$:			ld	A, (HL)
			cp	#" "
			jr	NC, 6$
			ld	A, #"."
6$:			call	conout
			inc	HL
			dec	C
			ld	A, #0H0F
			and	C
			jr	NZ, 5$
7$:			pop	BC
			pop	HL
			ret


hex2:			push	HL
			ld	HL, #REG_STR_BUF
			call	str_ltoah
			ld	HL, #REG_STR_BUF
			call	conout_asciiz
			pop	HL
			ret

hex1:			push	HL
			ld	HL, #REG_STR_BUF
			call	str_btoah
			ld	HL, #REG_STR_BUF
			call	conout_asciiz
			pop	HL
			ret

crlf:			push	HL
			ld	HL, #sz_crlf
			call	conout_asciiz
			pop	HL
			ret

conout:			push	HL
			push	DE
			push	BC
			push	AF
			ld	E, A
			ld	C, #BIOS_CONOUT
			call	5
			pop	AF
			pop	BC
			pop	DE
			pop	HL
			ret

condirin:		push	HL
			push	DE
			push	BC
			ld	C, #BIOS_CONDIR
			ld	E, #0HFF
			call	5
			pop	BC
			pop	DE
			pop	HL
			ret

conout_asciiz:		push	HL
			push	DE
			push	BC
			push	AF
2$:			ld	A, (HL)
			or	A
			jr	Z, 1$
			ld	E, A
			push	HL
			ld	C, #BIOS_CONOUT
			call	5
			pop	HL
			inc	HL
			jr	2$
1$:			pop	AF
			pop	BC
			pop	DE
			pop	HL
			ret

			; updates dump ptr and returns length in BC (default 256)
get_start_length:	call	str_ahtol
			jr	Z, 1$
			ld	(DUMP_PTR), DE
1$:			ld	A, (HL)
			or	A
			jr	Z, 2$
			cp	#","
			jp	NZ, main_decode_error
			inc	HL
			call	str_ahtol
			jp	Z, 2$
			ld	B, D
			ld	C, E
			jr	3$
2$:			ld	BC, #0H0100		;do a page
3$:			ret


CMD_DUMP:		call	get_start_length
			call	dumpN
			jp	main_command_loop

CMD_EXAMINE:		call	dump_regs
			jp	main_command_loop

CMD_GO:			call	get_start_length
			ld	SP, #ARCH_MEM_TRANS_STACK   ; new stack
			ld	HL, #return		; push a return address
			push	HL	
			ld	HL, (DUMP_PTR)		; jump to GO
			jp	(HL)

return:			ld	SP, #ARCH_MEM_USER_TOP; reset stack
			ld	HL, #sz_return_message
			call	crlf
			call	conout_asciiz
			call	crlf
			call	dump_regs
			jp	main_command_loop

CMD_MONITOR:		ld	A,#01
			call	bios_monitor
			jp	main_command_loop

CMD_READ:		xor	A
			ld	B, #3
			ld	HL, #READ_LEN
4$:			ld	(HL), A
			inc	HL
			djnz	4$
			ld	IX, #STR_BUF + 2
			ld	IY, #READ_LEN
			ld	DE, #s$read_msg
			ld	C, #BIOS_PRINTSTR
			call	5
			call	crlf
6$:			call	read_line
			ld	HL, #STR_BUF + 2
			call	str_ihextobytes
			jr	NC, 1$
			set	0, 2(IY)
			ld	HL, #sz_err
			jr	2$
1$:			ld	HL, #READ_STR_BUF
			ld	B, 1(IX)
			ld	C, 2(IX)
			push	HL
			call	str_ltoah
			pop	HL
			call	conout_asciiz
			ld	A, #" "
			call	conout
			ld	HL, #sz_ok
2$:			call	conout_asciiz
			call	crlf
			; check to see if it is an EOF marker
			ld	A, 3(IX)
			cp	#01
			jr	Z, 3$
			cp	#00
			jr	NZ, 6$
			ld	C, 0(IX)		; copy bytes to memory
			ld	B, #0
			ld	HL, #STR_BUF + 6
			ld	D, 1(IX)
			ld	E, 2(IX)
			ldir
			ld	A, 0(IY)
			add	3(IX)
			ld	0(IY), A
			jr	NZ, 6$
			inc	1(IY)
			jr	6$

3$:			bit	0, 2(IY)
			jr	Z, 5$
			ld	HL, #sz_err
			call	conout_asciiz
5$:			ld	HL, #STR_BUF
			ld	A, 1(IY)
			call	str_btoah
			ld	A, 0(IY)
			call	str_btoah
			call	crlf

			jp	main_command_loop


						
rom_loader:		; ROM numbered D to memory starting at HL
			; rom numbers are as specified in the IO register for ROM
			; i.e. inverted
			; each "rom" is 4k long
			; note this code must be relocatable it will be copied to ARCH_MEM_BIOS_WORK before being executed - all jumps should be to this area!
			; the rom must start with the rubric "ROMF" followed by the length of the "file"
			; AF, BC, DE, HL corrupted
			di				; disable interrupts we're going to unplug the ROM!
			push	HL
			ld	A, C
			push	AF
			cpl
			or	#0HC0
			out	(ARCH_IO_ROMAD), A
			pop	AF
			ld	(LOAD_ROM_ROM), A
			ld	HL, #ARCH_MEM_ROMBASE
			ld	DE, #ARCH_MEM_BIOS_WORK + (s_rom_loader - rom_loader)
			ld	B, #4
1$:			ld	A, (DE)
			cp	(HL)
			jr	NZ, 5$			; check header
			inc	HL
			inc	DE
			djnz	1$
			ld	C, (HL)			; load file length
			inc	HL
			ld	B, (HL)
			inc	HL
			pop	DE
3$:			ldi
			jp	PO, #ARCH_MEM_BIOS_WORK + (4$ - rom_loader) ; finished OK
			ld	A, L			; check to see if on a boundary, if so bump up rom pointer
			and	A
			jr	NZ, 3$
			ld	A, H
			and	A, #0H0F
			jr	NZ, 3$
			ld	A, (LOAD_ROM_ROM)
			inc	A
			ld	(LOAD_ROM_ROM), A
			cpl
			or	#0HC0
			out	(ARCH_IO_ROMAD), A
			ld	HL, #ARCH_MEM_ROMBASE
			jr	3$

5$:			pop	HL
			ld	A, #1
			jr	2$
4$:			xor	A
2$:			push	AF
			ld	A, #0HC0
			out	(ARCH_IO_ROMAD), A
			pop	AF
			ei
			ret
s_rom_loader:		.ascii "ROMF"
rom_loader_end:		


CMD_LOAD:		push	HL
			; copy the loader to the BIOS_WORKSPACE
			ld	BC, #(rom_loader_end - rom_loader)
			ld	DE, #ARCH_MEM_BIOS_WORK
			ld	HL, #rom_loader
			ldir
			pop	HL
			call	get_start_length
			ld	HL, (DUMP_PTR)
			call	ARCH_MEM_BIOS_WORK
			and	A
			jp	Z, main_command_loop
			ld	HL, #sz_rom_err
			call	conout_asciiz
			call	crlf
			jp	main_command_loop
			
		

sz_welcome:		.ascii "Z80 Computer"
			.db 13,10
			.ascii "Dominic Beesley"
sz_crlf:		.db 13,10,0
sz_AF:			.asciz " AF="
sz_BC:			.asciz " BC="
sz_DE:			.asciz " DE="
sz_HL:			.asciz " HL="
sz_IX:			.asciz " IX="
sz_IY:			.asciz " IY="
sz_SP:			.asciz " SP="
sz_FLAGS:		.ascii "SZ5H3PNC"
sz_SEP:			.asciz " : "

s$message:		.ascii "Hello now here is a message from BIOS$"

s$read_msg:		.ascii "Reading Intel Hex from console, type intel hex or ctrl-c to abort$"

sz_ok:			.asciz "OK"
sz_err:			.asciz "Error"
sz_return_message:	.asciz "Returned from user program"
sz_rom_err:		.asciz "Not a ROM file"

cmd_table:
			.db "D"
			.dw CMD_DUMP
			.dw 0

			.db "X"
			.dw CMD_EXAMINE
			.dw 0

			.db "R"
			.dw CMD_READ
			.dw 0
			
			.db "G"
			.dw CMD_GO
			.dw 0

			.db "M"
			.dw CMD_MONITOR
			.dw 0

			.db "L"
			.dw CMD_LOAD
			.dw 0

			.db 0	;end of command table
