			.area CODE2

str_ltoah::		; converts number in BC to string at HL - hex, 0 terminated, HL points at 0 terminator
			; preserves all registers except HL
			
			push	AF
			push	BC
			push	DE


			ld	A, B
			call	hex
			ld	A, C
			call	hex

			xor	A
			ld	(HL), A

			pop	DE
			pop	BC
			pop	AF
			ret

str_btoah::		; converts number in A to string at HL - hex, 0 terminated, HL points at 0 terminator
			; preserves all registers except HL
			push	AF
			call	hex
			xor	A
			ld	(HL), A
			pop	AF
			ret


hex:			push	AF
			rra
			rra
			rra
			rra
			call	str_tohex
			ld	(HL), A
			inc	HL

			pop	AF
			call	str_tohex
			ld	(HL), A
			inc	HL
			ret



str_tohex::		; converts low nibble of A to hex char in A
			and	#0H0F
			cp	#0H0A

			jr	NC, 1$
			add	#"0"
			ret
1$:			add	#"A"-10
			ret

str_ahtol::		; conerts hex high bits discarded at HL to hex
			; returns DE = hex
			; HL points to next character
			; A corrupted
			; BC preserved
			; Z set if something read
			push	BC
			ld	B, #0
			ld	DE, #0
1$:			ld	A, (HL)
			call	str_ctoh
			jr	C, 3$
2$:			;shift low nibble of A into DE
			sla	E
			rl	D
			sla	E
			rl	D
			sla	E
			rl	D
			sla	E
			rl	D
			or	E	; add in value
			ld	E, A	
			inc	HL
			inc	B
			jr	1$
3$:			ld	A, B
			and	A
			pop	BC
			ret

str_ctoh::		; converts character in A to hex, returns with C set (and A corrupted) on error
			sub	#"0"
			ret	C
			cp	#10
			jr	C, 2$
			ld	A, (HL)
			or	#0H20
			sub	#"a" - 10
			ret	C
			cp	#16
			jr	C, 2$
			scf
			ret
2$:			; return success
			and	A		; clear carry
			ret

str_dctoh::		; converts two hex chars at (HL) to hex in Am returns with C set on error and HL pointing to error character
			; on success HL points at character after last one read
			push	BC
1$:			ld	A, (HL)
			call	str_ctoh
			jr	C, 3$
			;shift low nibble of A into DE
			sla	A
			sla	A
			sla	A
			sla	A
			inc	HL
			ld	B, A
			ld	A, (HL)
			call	str_ctoh
			jr	C, 3$
			inc	HL
			or	B			
3$:			pop	BC
			ret

str_ihextobytes::	; converts the 0 terminated intel hex record at HL into a stream of bytes
			; returns with C set if there is any problem
			; preserves all registers except AF
			; if C is set buffer may be corrupt
			; ignores trailing characters after checksum
			push	HL
			push	DE
			push	BC

			ld	D, H
			ld	E, L
			ld	C, #0		; checksum
1$:			ld	A, (HL)		; skip whitespace
			cp	#33
			jr	C, 1$	
			cp	#":"		; check for record start mark
			jr	NZ, 4$
			inc	HL
			call	str_dctoh	; get length 
			jr	C, 4$
			ld	(DE), A
			push	AF
			add	C
			ld	C, A
			pop	AF
			add	#4		; add 4 for address, type and checksum
			ld	B, A		; byte counter
			inc	DE
2$:			call	str_dctoh	; get next
			jr	C, 4$
			ld	(DE), A
			add	C
			ld	C, A
			inc	DE
			djnz	2$
			; check the last character is the correct checksum
			jr	NZ, 4$	; it should add to give 0H100 i.e. 0
5$:			; return with no error
			and	A
6$:			pop	BC
			pop	DE
			pop	HL			
			ret			

4$:			; return with an error condition flagged
			scf
			jr	6$
